###############
# Re-model Stevens et al. 2004 using subset vars [Model 1]: vascular plants only
# O.L. Pescott, olipes@ceh.ac.uk
################
#options(repos = c(getOption("repos"), INLA="https://inla.r-inla-download.org/R/stable"))
#install.packages("INLA", repos=c(getOption("repos"), INLA="https://inla.r-inla-download.org/R/stable"), dep=TRUE)
rm(list=ls())
library(INLA)
library(fields)
library(ggplot2)
library(lme4)
library(car)

load(file = "data/stevensDatFinal_Mod1.Rdata")
load(file = "data/mesh1_Mod1.Rdata")
load(file = "data/monads_Mod1.Rdata")
load(file = "data/UK.Rdata")
plot(mesh1) # 

# Define the projection matrix using the locations of data points
coords1 <- as.matrix(allDatSubset[,c("EASTING2", "NORTHING2")])
AN1 <- inla.spde.make.A(mesh1, loc=coords1)
# Build the SPDE model:
spde1 <- inla.spde2.matern(mesh1)

# Create the data "stack" for the model estimation:
##############################################
# species richness here is only vascular plants
# uncentered
##############################################
stkSD <- inla.stack(data=list(sprich = allDatSubset$sprichness, link = "log"), # for count data
                   A=list(AN1,1), # projection matrix
                   effects=list(nodes=1:spde1$n.spde, # SPDE model
                                data.frame(intercept = rep(1, length(allDatSubset$sprichness)), # vascular plants
                                           tot_ndep = allDatSubset$Ndep,
                                           alt = allDatSubset$alti_SCD,
                                           minT = allDatSubset$minT,
                                           maxT = allDatSubset$maxT,
                                           precip = allDatSubset$rain_SCD,
                                           top_ph = allDatSubset$topsoil.pH_SCD,
                                           al_con = allDatSubset$Al_SCD,
                                           sulph = allDatSubset$Sdep_kg,
                                           graze_med = allDatSubset$grazingmedium,
                                           graze_high = allDatSubset$grazinghigh,
                                           monad = allDatSubset$monad)), 
                   tag='est')

### check VIFs ###
car::vif(glmer(sprichness ~ Ndep + alti_SCD + minT + maxT + rain_SCD + topsoil.pH_SCD + Al_SCD + Sdep_kg + grazing
               + (1 | monad), data = allDatSubset, family = "poisson"))
###

# Formula 3: with monad
formula3 <- sprich ~ -1 + intercept + tot_ndep + alt + minT + maxT + precip + sulph + graze_med + graze_high + top_ph + al_con +
  f(monad, model = "iid") +
  f(nodes, model = spde1) # 
# Formula 3a: without monad
formula3a <- sprich ~ -1 + intercept + tot_ndep + alt + minT + maxT + precip + sulph + graze_med + graze_high + top_ph + al_con +
  f(nodes, model = spde1) # 
# Formula 3b: without mesh
formula3b <- sprich ~ -1 + intercept + tot_ndep + alt + minT + maxT + precip + sulph + graze_med + graze_high + top_ph + al_con +
  f(monad, model = "iid")  # 

ptm=proc.time()
model3 <- inla(formula3, family = "poisson",
               data = inla.stack.data(stkSD), # not centered
               control.predictor = list(A = inla.stack.A(stkSD)),
               control.fixed = list(expand.factor.strategy = "model.matrix", compute = T),
               control.compute = list(dic = TRUE))
proc.time()-ptm   #
summary(model3) # Ndep effect -0.0092, 2.5% -0.022 97.5% 0.0048

ptm=proc.time()
model3a <- inla(formula3a, family = "poisson",
               data = inla.stack.data(stkSD), # not centered
               control.predictor = list(A = inla.stack.A(stkSD)),
               control.fixed = list(expand.factor.strategy = "model.matrix", compute = T),
               control.compute = list(dic = TRUE))
proc.time()-ptm   #
summary(model3a)

ptm=proc.time()
model3b <- inla(formula3b, family = "poisson",
                data = inla.stack.data(stkSD), # not centered
                control.predictor = list(A = inla.stack.A(stkSD)),
                control.fixed = list(expand.factor.strategy = "model.matrix", compute = T),
                control.compute = list(dic = TRUE))
proc.time()-ptm   #
summary(model3b)

mod3FixedDf <- do.call(rbind, lapply(lapply(model3$marginals.fixed, function(x) inla.zmarginal(x,silent=TRUE)), function(x) as.data.frame(x)))
round(mod3FixedDf, 4)

######################
## Coefplot type plot
######################
mod3FixedDf$vars <- c("intercept", "Ndep", "Max. alt.", "Mean min\n daily temp.", "Mean max\n daily temp.", "Precip.", 
                      "Sdep", "Grazing: Med.", "Grazing: High", "pH", "Al.")
#tiff("outputs/modelSEA04_MEA10comp_coefs_woBryos.tiff", height=1500, width=1500, res = 300) #
#pdf("outputs/modelSEA04_MEA10comp_coefs_woBryos.pdf") 
fig1b <- ggplot(mod3FixedDf[c(2:11),], aes(vars, quant0.5)) + 
  geom_hline(yintercept=0, lty=2, lwd=1, colour="grey50") +
  geom_errorbar(aes(ymin = quant0.025, ymax = quant0.975), 
                lwd=1.5, colour="grey", width=0) +
  geom_errorbar(aes(ymin = quant0.25, ymax = quant0.75), 
                lwd=2.5, colour="gray0", width=0) +
  geom_point(size=2.0, pch=21, fill="white") +
  labs(y = "Regression coefficient estimates") +
  theme_bw() +
  theme(axis.title.y=element_blank() ) +
  coord_flip() # export 558 x 509
#dev.off()
######

# Fixed effects - Intercept:
exp(c(model3$summary.fixed$`0.025quant`[1], model3$summary.fixed$`0.5quant`[1], model3$summary.fixed$`0.975quant`[1]))
# Fixed effects - Ndep:
exp(c(model3$summary.fixed$`0.025quant`[2], model3$summary.fixed$`mean`[2], model3$summary.fixed$`0.975quant`[2]))

# Spatial variance:
model3.field <- inla.spde2.result(inla=model3,
                                  name="nodes", spde=spde1, do.transf=TRUE)
names(model3.field)
var.nom.marg = model3.field$marginals.variance.nominal[[1]]
var.nom.m1 = inla.emarginal(function(x) x, var.nom.marg)
var.nom.m2 = inla.emarginal(function(x) x^2, var.nom.marg)
var.nom.stdev = sqrt(var.nom.m2 - var.nom.m1^2)
var.nom.quantiles = inla.qmarginal(c(0.025, 0.5, 0.975), var.nom.marg)
print(var.nom.quantiles)
var.RRhet<-inla.emarginal(function(x) 1/x, model3$marginals.hyper$"Precision for monad")
var.nom.m1/(var.nom.m1 + var.RRhet) # fraction of variance explained by mesh = 99.0

## DIC:
#model2$dic$dic # 1643.17 # without monad RE
model3$dic$dic # 1643.21 # with monad RE
model3a$dic$dic # 1644.49 # without monad RE
model3b$dic$dic # 1650.65 # without mesh RE
model3a$dic$dic - model3$dic$dic # monad effect = 1.27
model3b$dic$dic - model3$dic$dic # mesh effect = 7.44

# Project the mesh onto a structured grid 500x1000 cells in dimension
proj1 <- inla.mesh.projector(mesh1, ylim=range(monad$NORTHING2, na.rm=TRUE),
                             xlim=range(monad$EASTING2, na.rm=TRUE),
                             dims=c(500,1000))

# Pull out the mean of the random field for the model
x3mean1 <- inla.mesh.project(proj1, model3$summary.random$nodes$mean)
x3sd1 <- inla.mesh.project(proj1, model3$summary.random$nodes$sd)

#tiff("outputs/modelSEA04_MEA10comp_RFs_woBryos_FINAL.tiff", height=750, width=1000) # need to fix right hand side margin here
# Fig. A1.3
par(mfrow=c(1,2), mar=c(4.1,4.1,4.1,5.0))
fields::image.plot(seq(min(monad$EASTING2, na.rm=TRUE)*1000, max(monad$EASTING2, na.rm=TRUE)*1000, len=500),
                   seq(min(monad$NORTHING2, na.rm=TRUE)*1000, max(monad$NORTHING2, na.rm=TRUE)*1000, len=1000),
                   #exp(xmean1), 
                   x3mean1,
                   col.regions=tim.colors(),
                   xlab='', 
                   ylab='', 
                   scales=list(draw = FALSE),
                   main="Mean of RF",
                   asp=1)
plot(UK[[1]], add = TRUE, col = "transparent", border = "grey") #
points(cbind(allDatSubset$EASTING2*1000, allDatSubset$NORTHING2*1000), pch = "X", cex = 0.5, col = "white") # add UK# add UK
#plot(UK[[2]], add = TRUE) # add Ireland

fields::image.plot(seq(min(monad$EASTING2, na.rm=TRUE)*1000, max(monad$EASTING2, na.rm=TRUE)*1000, len=500),
                   seq(min(monad$NORTHING2, na.rm=TRUE)*1000, max(monad$NORTHING2, na.rm=TRUE)*1000, len=1000),
                   x3sd1, 
                   col.regions=tim.colors(),
                   xlab='', 
                   ylab='', 
                   scales=list(draw = FALSE),
                   axis.args=list(at = c(0.09,0.10,0.11,0.12,0.13,0.14,0.15), labels = c(0.09,0.10,0.11,0.12,0.13,0.14,0.15)),
                   main="SD of RF",
                   asp=1)
plot(UK[[1]], add = TRUE, col = "transparent", border = "grey") #
points(cbind(allDatSubset$EASTING2*1000, allDatSubset$NORTHING2*1000), pch = "X", cex = 0.5, col = "white") # add UK
#plot(UK[[2]], add = TRUE) # add Ireland
#dev.off() # Warning, don't zoom on these in RStudio as it messes up the alignment
#save.image(file = "outputs/20180430_wksp1.Rdata") # save workspace (27 04 2018)

#######################################
### partial effects
#######################################
weights <- diag(inla.mesh.fem(mesh1)$c0)
weights2 <- weights/(sum(weights))

# Create (shorter) vector of standardised ndep values for which we want to make predictions
ndepRange <- round(quantile(allDatSubset$Ndep, probs=seq(from=0, to=1, by=0.1)), 2)
attr(ndepRange, "names") <- NULL
# apply inla.make.lincomb across values of ndep
lc_list<- list()

for (i in 1:length(ndepRange)) { lc_list[i] <- inla.make.lincomb(tot_ndep = ndepRange[i])}
names(lc_list) <- c("lc1","lc2","lc3","lc4","lc5","lc6","lc7","lc8","lc9","lc10","lc11")
ndepOut <- data.frame(out = matrix(ncol = 3, nrow = length(ndepRange)))

# loop model across values of ndep for which predictions are desired
## SLOW! Creates predictions (linear combinations) for 21 models
#for(i in 1:length(lc_list)){
modelI3 <- inla(formula3, family = "poisson",
                  data = inla.stack.data(stkSD),
                  #lincomb = lc_list[i],
                  lincomb = lc_list,
                  control.predictor = list(A = inla.stack.A(stkSD), compute = T), 
                  control.fixed = list(expand.factor.strategy = "inla"),
                  control.inla = list(lincomb.derived.only=TRUE),
                  #control.inla = list(lincomb.derived.only=F), # gives almost exactly the same answer
                  control.compute = list(dic = TRUE))
summary(modelI3)
#  ndepOut[i,] <- exp(modelI3$summary.lincomb.derived[,4:6]) # back transform responses here
#} # END
ndepOut <- exp(modelI3$summary.lincomb.derived[,4:6]) # back transform responses here
names(ndepOut) <- c("out.1","out.2","out.3")
ndepOut$ndep <- ndepRange # add Ndep data for plotting convenience
head(ndepOut)

#tiff("outputs/Ndep_marg_model3_finalLCs.tiff", height=1500, width=1500, res = 300) #
fig2b <- ggplot(ndepOut) + geom_line(aes(x=ndep, y=out.2, colour = "sin")) +
  geom_ribbon(aes(ymin=out.1, ymax=out.3, x=ndep, fill = "band"), alpha = 0.3) +
  #ggtitle("Ndep v. Spp. richness: marginal response SEA04, vascular plants") +
  xlab("Ndep") +
  ylab("Predicted species richness") +
  scale_x_continuous(limits = c(7,42)) +
  scale_y_continuous(limits = c(0,5)) +
  scale_colour_manual("",values="red") +
  scale_fill_manual("",values="red") +
  theme(legend.position="none",
        axis.line = element_line(color = 'black'),
        text = element_text(size=14),
        axis.text.x = element_text(size=12),
        axis.text.y = element_text(size=12)
  )
#dev.off() 
#ndepOut[c(1,11),] 
