###############
# Re-model Stevens et al. 2004 using subset vars [Model 1]: vascular plants + bryos
# # O.L. Pescott, olipes@ceh.ac.uk
################
#options(repos = c(getOption("repos"), INLA="https://inla.r-inla-download.org/R/stable"))
#install.packages("INLA", repos=c(getOption("repos"), INLA="https://inla.r-inla-download.org/R/stable"), dep=TRUE)
rm(list=ls())
library(INLA)
library(fields)
library(ggplot2)
library(lme4)
library(car)

load(file = "data/stevensDatFinal_Mod1.Rdata")
load(file = "data/mesh1_Mod1.Rdata")
load(file = "data/monads_Mod1.Rdata")
load(file = "data/UK.Rdata")
plot(mesh1) # 

# Define the projection matrix using the locations of data points
coords1 <- as.matrix(allDatSubset[,c("EASTING2", "NORTHING2")])
AN1 <- inla.spde.make.A(mesh1, loc=coords1)

# Build the SPDE model:
spde1 <- inla.spde2.matern(mesh1)

# Create the data "stack" for the model estimation:
#########################################
# species richness here is including bryos
# uncentered
#########################################
stkSD <- inla.stack(data=list(sprich = allDatSubset$sprichnessIncBryos, link = "log"), # for count data
                   A=list(AN1,1), # projection matrix
                   effects=list(nodes=1:spde1$n.spde, # SPDE model
                                data.frame(intercept = rep(1, length(allDatSubset$sprichnessIncBryos)), # include bryos
                                           tot_ndep = allDatSubset$Ndep,
                                           alt = allDatSubset$alti_SCD,
                                           minT = allDatSubset$minT,
                                           maxT = allDatSubset$maxT,
                                           precip = allDatSubset$rain_SCD,
                                           top_ph = allDatSubset$topsoil.pH_SCD,
                                           al_con = allDatSubset$Al_SCD,
                                           graze_med = allDatSubset$grazingmedium,
                                           graze_high = allDatSubset$grazinghigh,
                                           sulp = allDatSubset$Sdep_kg,
                                           monad = allDatSubset$monad)), 
                   tag='est')

### check VIFs ###
car::vif(glmer(sprichnessIncBryos ~ Ndep + alti_SCD + minT + maxT + rain_SCD + topsoil.pH_SCD + Al_SCD + Sdep_kg + grazing
               + (1 | monad), data = allDatSubset, family = "poisson"))
###

# Formula 4: Reduced model, inc bryos
formula4 <- sprich ~ -1 + intercept + tot_ndep + alt + minT + maxT + precip + top_ph + al_con + graze_med + graze_high + sulp +
  f(monad, model = "iid") +
  f(nodes, model = spde1)
# without monad random effect
formula4a <- sprich ~ -1 + intercept + tot_ndep + alt + minT + maxT + precip + top_ph + al_con + graze_med + graze_high + sulp +
  f(nodes, model = spde1)
# without mmesh random effect
formula4b <- sprich ~ -1 + intercept + tot_ndep + alt + minT + maxT + precip + top_ph + al_con + graze_med + graze_high + sulp +
  f(monad, model = "iid")
# without either
formula4x <- sprich ~ -1 + intercept + tot_ndep + alt + minT + maxT + precip + top_ph + al_con + graze_med + graze_high + sulp

# Model a:
ptm=proc.time()
model4 <- inla(formula4, family = "poisson",
               data = inla.stack.data(stkSD),
               control.predictor = list(A = inla.stack.A(stkSD)),
               control.fixed = list(expand.factor.strategy = "model.matrix", compute = T),
               control.compute = list(dic = TRUE))
proc.time()-ptm   #
summary(model4)
# Model 4a:
ptm=proc.time()
model4a <- inla(formula4a, family = "poisson",
               data = inla.stack.data(stkSD),
               control.predictor = list(A = inla.stack.A(stkSD)),
               control.fixed = list(expand.factor.strategy = "model.matrix", compute = T),
               control.compute = list(dic = TRUE))
proc.time()-ptm   #
# Model 4b:
ptm=proc.time()
model4b <- inla(formula4b, family = "poisson",
                data = inla.stack.data(stkSD),
                control.predictor = list(A = inla.stack.A(stkSD)),
                control.fixed = list(expand.factor.strategy = "model.matrix", compute = T),
                control.compute = list(dic = TRUE))
proc.time()-ptm   #
# Model 4x
ptm=proc.time()
model4x <- inla(formula4x, family = "poisson",
                data = inla.stack.data(stkSD),
                control.predictor = list(A = inla.stack.A(stkSD)),
                control.fixed = list(expand.factor.strategy = "model.matrix", compute = T),
                control.compute = list(dic = TRUE))
proc.time()-ptm   #

# this strategy is easier for later use in ggplot2
mod4FixedDf <- do.call(rbind, lapply(lapply(model4$marginals.fixed, function(x) inla.zmarginal(x,silent=TRUE)), function(x) as.data.frame(x)))
round(mod4FixedDf, 4)
plot(model4)
plot(model4, single = TRUE)

# without intercept
mod4FixedDf$vars <- c("intercept", "Ndep", "Max. alt.", "Mean min\n daily temp.", "Mean max\n daily temp.", "Precip.", 
                      "Topsoil pH", "Topsoil Al", "Grazing: Med.", "Grazing: High", "Sdep")
#tiff("outputs/modelSEA04_MEA10comp_coefs_withBryos.tiff", height=1500, width=1500, res = 300) # need to fix right hand side margin here
figA2.2 <- ggplot(mod4FixedDf[c(2:11),], aes(vars, quant0.5)) + 
  geom_hline(yintercept=0, lty=2, lwd=1, colour="grey50") +
  geom_errorbar(aes(ymin = quant0.025, ymax = quant0.975), 
                lwd=1.5, colour="grey", width=0) +
  geom_errorbar(aes(ymin = quant0.25, ymax = quant0.75), 
                lwd=2.5, colour="gray0", width=0) +
  geom_point(size=2.0, pch=21, fill="white") +
  labs(y = "Regression coefficient estimates") +
  theme_bw() +
  theme(axis.title.y=element_blank() ) +
  coord_flip() # export 558 x 509
#dev.off()

# Fixed effects - Intercept:
exp(c(model4$summary.fixed$`0.025quant`[1], model4$summary.fixed$`0.5quant`[1], model4$summary.fixed$`0.975quant`[1]))
# Fixed effects - Ndep:
exp(c(model4$summary.fixed$`0.025quant`[2], model4$summary.fixed$`mean`[2], model4$summary.fixed$`0.975quant`[2]))

# Spatial variance:
model4.field <- inla.spde2.result(inla=model4,
                                  name="nodes", spde=spde1, do.transf=TRUE)
names(model4.field)
var.nom.marg = model4.field$marginals.variance.nominal[[1]]
var.nom.m1 = inla.emarginal(function(x) x, var.nom.marg)
var.nom.m2 = inla.emarginal(function(x) x^2, var.nom.marg)
var.nom.stdev = sqrt(var.nom.m2 - var.nom.m1^2)
var.nom.quantiles = inla.qmarginal(c(0.025, 0.5, 0.975), var.nom.marg)
print(var.nom.quantiles)
var.RRhet<-inla.emarginal(function(x) 1/x, model4$marginals.hyper$"Precision for monad")
var.nom.m1/(var.nom.m1 + var.RRhet) # fraction of variance explained by mesh = 99.7

# DIC:
model4$dic$dic # 1712.65  # full model
model4a$dic$dic # 1708.96  # without monad RE
model4b$dic$dic # 1711.57  # without mesh RE
model4a$dic$dic - model4$dic$dic # monad effect = -3.68
model4b$dic$dic - model4$dic$dic # mesh effect = -1.07
model4x$dic$dic # 1732.68
model4x$dic$dic - model4$dic$dic # no spatial effects = 20

# Project the mesh onto a structured grid 500x1000 cells in dimension
proj1 <- inla.mesh.projector(mesh1, ylim=range(monad$NORTHING2, na.rm=TRUE),
                             xlim=range(monad$EASTING2, na.rm=TRUE),
                             dims=c(500,1000))

# Pull out the mean of the random field for the model
x4mean1 <- inla.mesh.project(proj1, model4$summary.random$nodes$mean)
x4sd1 <- inla.mesh.project(proj1, model4$summary.random$nodes$sd)

#tiff("outputs/modelSEA04_MEA10comp_RFs_withBryos_FINAL.tiff", height=750, width=1000) # need to fix right hand side margin here
# FigA1.4
par(mfrow=c(1,2), mar=c(4.1,4.1,4.1,5.0))
fields::image.plot(seq(min(monad$EASTING2, na.rm=TRUE)*1000, max(monad$EASTING2, na.rm=TRUE)*1000, len=500),
                   seq(min(monad$NORTHING2, na.rm=TRUE)*1000, max(monad$NORTHING2, na.rm=TRUE)*1000, len=1000),
                   #exp(xmean1), 
                   x4mean1,
                   col.regions=tim.colors(),
                   xlab='', 
                   ylab='', 
                   scales=list(draw = FALSE),
                   main="Mean of RF",
                   asp=1)
plot(UK[[1]], add = TRUE, col = "transparent", border = "grey") #
points(cbind(allDatSubset$EASTING2*1000, allDatSubset$NORTHING2*1000), pch = "X", cex = 0.5, col = "white") # add UK
#plot(UK[[2]], add = TRUE) # add Ireland

fields::image.plot(seq(min(monad$EASTING2, na.rm=TRUE)*1000, max(monad$EASTING2, na.rm=TRUE)*1000, len=500),
                   seq(min(monad$NORTHING2, na.rm=TRUE)*1000, max(monad$NORTHING2, na.rm=TRUE)*1000, len=1000),
                   x4sd1, 
                   col.regions=tim.colors(),
                   xlab='', 
                   ylab='', 
                   scales=list(draw = FALSE),
                   main="SD of RF",
                   asp=1)
plot(UK[[1]], add = TRUE, col = "transparent", border = "grey") #
points(cbind(allDatSubset$EASTING2*1000, allDatSubset$NORTHING2*1000), pch = "X", cex = 0.5, col = "white") # add UK
#plot(UK[[2]], add = TRUE) # add Ireland
#dev.off() # Warning, don't zoom on these in RStudio as it messes up the alignment
##END