###############
# Re-model Stevens et al. 2004 using all vars [Model 2]: with Bryos
# O.L. Pescott, olipes@ceh.ac.uk
################
#options(repos = c(getOption("repos"), INLA="https://inla.r-inla-download.org/R/stable"))
#install.packages("INLA", repos=c(getOption("repos"), INLA="https://inla.r-inla-download.org/R/stable"), dep=TRUE)
rm(list=ls())
library(INLA)
library(fields)
library(ggplot2)
library(lme4)
library(car)

load(file = "data/stevensDatFinal_Mod2.Rdata")
load(file = "data/mesh_Mod2.Rdata")
load(file = "data/monads_Mod2.Rdata")
load(file = "data/UK.Rdata")
plot(mesh1) # 

# Define the projection matrix using the locations of data points
coords1 <- as.matrix(allDatSubset[,c("EASTING2", "NORTHING2")])
AN1 <- inla.spde.make.A(mesh1, loc=coords1)

# Build the SPDE model:
spde1 <- inla.spde2.matern(mesh1)

# Create the data "stack" for the model estimation:
##############################################
# species richness here is including bryophytes
##############################################
# uncentered
stk6 <- inla.stack(data=list(sprich = allDatSubset$sprichnessIncBryos, link = "log"), # for count data
                   A=list(AN1,1), # projection matrix
                   effects=list(nodes=1:spde1$n.spde, # SPDE model
                                data.frame(intercept = rep(1, length(allDatSubset$sprichnessIncBryos)), # including bryos
                                           tot_ndep = allDatSubset$Ndep,
                                           alt = allDatSubset$alti_SCD,
                                           clim1 = allDatSubset$PC1,
                                           clim2 = allDatSubset$PC2,
                                           precip = allDatSubset$rain_SCD,
                                           top_ph = allDatSubset$topsoil.pH_SCD,
                                           graze_med = allDatSubset$grazingmedium,
                                           graze_high = allDatSubset$grazinghigh,
                                           smd = allDatSubset$smd_SCD,
                                           slope = allDatSubset$inclination_SCD,
                                           CN = allDatSubset$C.N,
                                           percN = allDatSubset$N,
                                           sulp = allDatSubset$Sdep_kg,
                                           monad = allDatSubset$monad)), 
                   tag='est')

### check VIFs ###
car::vif(glmer(sprichnessIncBryos ~ Ndep + alti_SCD + PC1 + PC2 + rain_SCD + topsoil.pH_SCD + grazing +
                 smd_SCD + inclination_SCD + C.N + N + Sdep_kg
               + (1 | monad), data = allDatSubset, family = "poisson"))
###

# Formula 6
formula6 <- sprich ~ -1 + intercept + tot_ndep + alt + clim1 + clim2 + precip + top_ph + graze_med + graze_high + 
  smd + slope + CN + percN + sulp +
  f(monad, model = "iid") + # with or without monad RE
  f(nodes, model = spde1) # 
formula6a <- sprich ~ -1 + intercept + tot_ndep + alt + clim1 + clim2 + precip + top_ph + graze_med + graze_high + 
  smd + slope + CN + percN + sulp +
  f(nodes, model = spde1) # 
formula6b <- sprich ~ -1 + intercept + tot_ndep + alt + clim1 + clim2 + precip + top_ph + graze_med + graze_high + 
  smd + slope + CN + percN + sulp +
  f(monad, model = "iid")  # 

# Model 6:
ptm=proc.time()
model6 <- inla(formula6, family = "poisson",
               data = inla.stack.data(stk6),
               control.predictor = list(A = inla.stack.A(stk6)),
               control.fixed = list(expand.factor.strategy = "model.matrix", compute = T),
               control.compute = list(dic = TRUE))
proc.time()-ptm   
summary(model6)
# without monad effect
ptm=proc.time()
model6a <- inla(formula6a, family = "poisson",
                 data = inla.stack.data(stk6),
                 control.predictor = list(A = inla.stack.A(stk6)),
                 control.fixed = list(expand.factor.strategy = "model.matrix", compute = T),
                 control.compute = list(dic = TRUE))
proc.time()-ptm   #
# without mesh effect
ptm=proc.time()
model6b <- inla(formula6b, family = "poisson",
                data = inla.stack.data(stk6),
                control.predictor = list(A = inla.stack.A(stk6)),
                control.fixed = list(expand.factor.strategy = "model.matrix", compute = T),
                control.compute = list(dic = TRUE))
proc.time()-ptm   #

# this strategy is easier for later use in ggplot2
mod6FixedDf <- do.call(rbind, lapply(lapply(model6$marginals.fixed, function(x) inla.zmarginal(x,silent=TRUE)), function(x) as.data.frame(x)))
#round(mod5FixedDf, 4)
#plot(model5)
#plot(model5, single = TRUE)

######################
## Coefplot type plot
######################
# model coefs
mod6FixedDf$vars <- c("intercept", "Ndep", "Max. alt.","Climate PC1", "Climate PC2", "Precip.", "Topsoil pH", "Grazing: Med.", "Grazing: High", 
                      "SMD", "Slope", "C:N", "Soil %N", "Sdep")
#tiff("outputs/modelSEA04_allVars_coefs_withBryos.tiff", height=1500, width=1500, res = 300) # need to fix right hand side margin here
figA2.3 <- ggplot(mod6FixedDf[c(2:14),], aes(vars, quant0.5)) + 
  geom_hline(yintercept=0, lty=2, lwd=1, colour="grey50") +
  geom_errorbar(aes(ymin = quant0.025, ymax = quant0.975), 
                lwd=1.5, colour="grey", width=0) +
  geom_errorbar(aes(ymin = quant0.25, ymax = quant0.75), 
                lwd=2.5, colour="gray0", width=0) +
  geom_point(size=2.0, pch=21, fill="white") +
  labs(y = "Regression coefficient estimates") +
  theme_bw() +
  theme(axis.title.y=element_blank() ) +
  coord_flip() # export 558 x 509
#dev.off()

# Fixed effects - Intercept:
exp(c(model6$summary.fixed$`0.025quant`[1], model6$summary.fixed$`0.5quant`[1], model6$summary.fixed$`0.975quant`[1]))
# Fixed effects - Ndep:
exp(c(model6$summary.fixed$`0.025quant`[2], model6$summary.fixed$`mean`[2], model6$summary.fixed$`0.975quant`[2]))

# Spatial variance:
model6.field <- inla.spde2.result(inla=model6,
                                  name="nodes", spde=spde1, do.transf=TRUE)
names(model6.field)
var.nom.marg = model6.field$marginals.variance.nominal[[1]]
var.nom.m1 = inla.emarginal(function(x) x, var.nom.marg)
var.nom.m2 = inla.emarginal(function(x) x^2, var.nom.marg)
var.nom.stdev = sqrt(var.nom.m2 - var.nom.m1^2)
var.nom.quantiles = inla.qmarginal(c(0.025, 0.5, 0.975), var.nom.marg)
print(var.nom.quantiles)
var.RRhet<-inla.emarginal(function(x) 1/x, model6$marginals.hyper$"Precision for monad")
var.nom.m1/(var.nom.m1 + var.RRhet) # fraction of variance explained by mesh = 99.3

# DIC:
model6$dic$dic # 1704.58 # 
model6a$dic$dic # 1706.61 # without monad RE
model6b$dic$dic # 1715.17 # without mesh RE
model6a$dic$dic - model6$dic$dic # 2.0
model6b$dic$dic - model6$dic$dic # 2.0

# Project the mesh onto a structured grid 500x1000 cells in dimension
proj1 <- inla.mesh.projector(mesh1, ylim=range(monad$NORTHING2, na.rm=TRUE),
                             xlim=range(monad$EASTING2, na.rm=TRUE),
                             dims=c(500,1000))

# Pull out the mean of the random field for the model
x6mean1 <- inla.mesh.project(proj1, model6$summary.random$nodes$mean)
x6sd1 <- inla.mesh.project(proj1, model6$summary.random$nodes$sd)

#tiff("outputs/modelSEA04_allVars_RFs_withBryos_FINAL.tiff", height=750, width=1000) # need to fix right hand side margin here
# Fig. A1.6
par(mfrow=c(1,2), mar=c(4.1,4.1,4.1,5.0))
fields::image.plot(seq(min(monad$EASTING2, na.rm=TRUE)*1000, max(monad$EASTING2, na.rm=TRUE)*1000, len=500),
                   seq(min(monad$NORTHING2, na.rm=TRUE)*1000, max(monad$NORTHING2, na.rm=TRUE)*1000, len=1000),
                   #exp(xmean1), 
                   x6mean1,
                   col.regions=tim.colors(),
                   xlab='', 
                   ylab='', 
                   scales=list(draw = FALSE),
                   main="Mean of RF",
                   asp=1)
plot(UK[[1]], add = TRUE, col = "transparent", border = "grey") # 
points(cbind(allDatSubset$EASTING2*1000, allDatSubset$NORTHING2*1000), pch = "X", cex = 0.5, col = "white") # add UK
#plot(UK[[2]], add = TRUE) # add Ireland

fields::image.plot(seq(min(monad$EASTING2, na.rm=TRUE)*1000, max(monad$EASTING2, na.rm=TRUE)*1000, len=500),
                   seq(min(monad$NORTHING2, na.rm=TRUE)*1000, max(monad$NORTHING2, na.rm=TRUE)*1000, len=1000),
                   x6sd1, 
                   col.regions=tim.colors(),
                   xlab='', 
                   ylab='', 
                   scales=list(draw = FALSE),
                   main="SD of RF",
                   asp=1)
plot(UK[[1]], add = TRUE, col = "transparent", border = "grey") # 
points(cbind(allDatSubset$EASTING2*1000, allDatSubset$NORTHING2*1000), pch = "X", cex = 0.5, col = "white") # add UK
#dev.off() # Warning, don't zoom on these in RStudio as it messes up the alignment
